package main

import (
	"fmt"

	"github.com/alyu/configparser"
	"github.com/gophercloud/gophercloud/openstack/identity/v3/projects"
)

func main() {
	domainID := "614b64b5d4ce4e989ad0590ec56b021f" // mephi domain

	var osclient OpenstackIdentityClient
	osclient.client = initIdentityClient()

	config, err := configparser.Read("./project_and_roles.conf")
	checkErr(err)
	osclient.config = config

	// allProjects := osclient.getProjectsMap(domainID)
	// for _, project := range allProjects {
	// 	fmt.Println(project)
	// }
	// allUnits := getCleanUnitsMap(osclient.config)
	// for _, unit := range allUnits {
	// 	fmt.Println(*unit.BriefName)
	// }

	fmt.Println("Syncing projects...")
	osclient.syncProjects(domainID)
	// fmt.Println("Deleting projects...")
	// osclient.deleteAllProjects(domainID)
	//fmt.Println("Syncing roles...")
	//syncRoles(domainID)
}

func (osclient OpenstackIdentityClient) syncProjects(domainID string) {
	allUnits := getCleanUnitsMap(osclient.config)
	allProjects := osclient.getProjectsMap(domainID)

	for persNumber, unit := range allUnits {
		if _, ok := allProjects[persNumber]; ok {
			updateOpts := makeUpdateOpts(unit, domainID)
			projectID := allProjects[persNumber].ID
			updateResult := projects.Update(osclient.client, projectID, updateOpts)
			checkErr(updateResult.Err)
			fmt.Printf("Project \"%s\" updated\n", *unit.BriefName)
		} else {
			createOpts := makeCreateOpts(unit, domainID)
			createResult := projects.Create(osclient.client, createOpts)
			checkErr(createResult.Err)
			fmt.Printf("Project \"%s\" created\n", *unit.BriefName)
		}
	}
}

func (osclient OpenstackIdentityClient) createOrUpdateParentIDs(domainID string) {
	allProjects := osclient.getProjectsMap(domainID)

	for _, project := range allProjects {
		if project.Extra["ParentPersNumber"] == nil {
			continue
		}
		parentPersNumber := int(project.Extra["ParentPersNumber"].(float64))
		parent, ok := allProjects[parentPersNumber]
		if !ok {
			panic(fmt.Sprintf("Cannot find parent for %s", project.Name))
		}
		if project.ParentID != parent.ID {
			updateOpts := projects.UpdateOpts{
				ParentID: parent.ID,
			}
			updateResult := projects.Update(osclient.client, project.ID, updateOpts)
			checkErr(updateResult.Err)
			fmt.Printf("Parent ID for %s set", project.Name)
		}
	}
}

func (osclient OpenstackIdentityClient) deleteAllProjects(domainID string) {
	allProjects := osclient.getProjectsMap(domainID)

	index := 1
	amount := len(allProjects)
	for _, project := range allProjects {
		deleteResult := projects.Delete(osclient.client, project.ID)
		checkErr(deleteResult.ExtractErr())
		fmt.Printf("[%v/%v] Project \"%s\" deleted\n", index, amount, project.Name)
		index++
	}
}

func syncRoles() {

}
